"""
             Yasuharu Okamoto, 2019.02, 2019.03
"""
import csv
import matplotlib.pyplot as plt
import numpy as np
import pymc3 as pm
import arviz as az

fin_name = input('Data file name(*.csv) = ')
with open(fin_name, 'r') as f:
    data = [v for v in csv.reader(f)]

fout_name = input('Output file name(*.txt) = ')
fout = open(fout_name, 'w')
fout.write('Data file name = {}\n'.format(fin_name))

print('data = \n', data)

N = len(data) - 1
M = len(data[0]) - 1
Items = []
for j in range(M):
    Items.append(data[0][j + 1])
X = np.empty((N, M), dtype = int)
CaseID = []
for i in range(1, N + 1):
    CaseID.append(data[i][0])
    for j in range(M):
        X[i - 1][j] = int(data[i][j + 1])
 
print('X =')

fout.write('\nX =\n')
i = 0
for v in X:
    print(CaseID[i], ' ',end = '')
    fout.write('{} '.format(CaseID[i]))
    i += 1
    print(v)
    fout.write('{}\n'.format(v))
N = len(X)
print('N = ', N)
fout.write('\nN = {}\n'.format(N))
M = len(X[0])
print('M = ', M)
fout.write('M = {}\n'.format(M))

NStg = int(input('Number of stages = '))
print('NStg = ', NStg)

a_Diri = np.ones((M, NStg + 1))
print('a_Diri =\n', a_Diri)

a_coeff = np.zeros((NStg, NStg + 1))
for i in range(NStg):
	for j in range(i + 1):
		a_coeff[i][j] = 1
print('a_coeff =\n', a_coeff)

a_p = np.ones(NStg)
print('a_p =\n', a_p)

with pm.Model() as stage_model:
	prob_steps = pm.Dirichlet('prob_steps', a = a_Diri, shape = (M, NStg + 1))
	coeff = pm.math.constant(a_coeff, 'coeff')
	prob_response_ = pm.math.matrix_dot(coeff, prob_steps.T)
	prob_response = pm.Deterministic('prob_response', prob_response_) 
	p_stages = pm.Categorical('p_stages', p = a_p, shape = N)
	p_at_stage = pm.Deterministic('p_at_stage', prob_response[p_stages])
	y = pm.Bernoulli('y', p = prob_response[p_stages],  observed = X)
	
	trace = pm.sample()	

summary = pm.summary(trace)
print('summary...\n', summary)
fout.write('summary...\n{}'.format(summary))

ary_prob_response = trace['prob_response']
print('\n\nP(correct|item,stage)\n')
fout.write('\n\nP(correct|item,stage)\n\n')
print('{0: >10s}'.format(' '), end = '')
fout.write('{0: >10s}'.format(' '))
for i in range(NStg):
	print("{0:>10s}".format('Stage-{}'.format(i)), end = '')
	fout.write("{0:>10s}".format('Stage-{}'.format(i)))
print(' ')  
fout.write('\n')
for j in range(M):
	print('{0:>10s}'.format(Items[j]), end = '')
	fout.write('{0:>10s}'.format(Items[j]))
	for s in range(NStg):
		print('{0: >10.5f}'.format(ary_prob_response[:,s,j].mean()), end = '')
		fout.write('{0: >10.5f}'.format(ary_prob_response[:,s,j].mean()))
	print(' ')
	fout.write('\n')

ary_p_stages = trace['p_stages']
ary_p_stages = ary_p_stages.T
print("\nMean stages and their SD's\n")
fout.write("\nMean stages and their SD's\n\n")
print('{0: >11s}{1:>10s}{2:>10s}'.format(' ', 'Mean', 'SD'))
fout.write('{0: >11s}{1:>10s}{2:>10s}\n'.format(' ', 'Mean', 'SD'))
for i in range(len(ary_p_stages)):
	print('{0:>11s}{1:>10.3f}{2:>10.3f}'.format(CaseID[i],
			 ary_p_stages[i].mean(), ary_p_stages[i].std()))
	fout.write('{0:>11s}{1:>10.3f}{2:>10.3f}\n'.format(CaseID[i],
			 ary_p_stages[i].mean(), ary_p_stages[i].std()))
													
v_waic = az.waic(trace, scale = 'deviance')
print('\nWAIC statistics...\n', v_waic, '\n')
fout.write('\nWAIC statistics...\n{}\n\n'.format(v_waic))
print('\nWAIC = {0:.3f}  for the number of stages = {1}'.format(v_waic['waic'], NStg))
fout.write('\nWAIC = {0:.3f}  for the number of stages = {1}\n'.format(v_waic['waic'], NStg))

print('Output file name =', fout_name)
fout.close()
